"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const glob_1 = __importDefault(require("glob"));
const minimist_1 = __importDefault(require("minimist"));
const path_1 = __importDefault(require("path"));
const wbb_logger_1 = __importDefault(require("wbb-logger"));
const model_1 = __importDefault(require("./model"));
const ts_interface_generator_1 = __importDefault(require("./ts-interface-generator"));
const logger = new wbb_logger_1.default('generate-interfaces.ts');
const getInterfacePath = (modelPath) => modelPath.replace(/.nongo\.ts$/, '-obj.ts');
const createInterfaces = async (modelDir, placeholder) => {
    var _a;
    const modelPaths = glob_1.default
        .sync(path_1.default.join(modelDir, '/**/*.nongo.ts'))
        .map((p) => path_1.default.resolve(p));
    // Generate placeholder interfaces
    if (placeholder) {
        for (const modelPath of modelPaths) {
            fs_1.default.writeFileSync(getInterfacePath(modelPath), 'export default interface TempObj {\n  [key: string]: any;\n}\n');
        }
        return;
    }
    while (modelPaths.length) {
        const modelPath = modelPaths.shift();
        // Find out information about the models super class
        const modelCode = fs_1.default.readFileSync(modelPath, 'utf8');
        const classDef = modelCode.match(/ *export +default.+class.*/).shift();
        const superClass = classDef
            .split('extends')
            .pop()
            .replace(/\{/g, '')
            .split('<')
            .shift()
            .trim();
        const superClassImport = modelCode
            .match(new RegExp('.*import .*' + superClass + '[ ,}].*'))
            .shift();
        const superClassPath = path_1.default.resolve(path_1.default.dirname(modelPath), superClassImport
            .match(/'.*'|".*"/)
            .shift()
            .replace(/['"]/g, '') + '.ts');
        // If we haven't already processed the super class do it before this model
        const indexOfSP = modelPaths.indexOf(superClassPath);
        if (indexOfSP >= 0) {
            modelPaths.splice(indexOfSP, 1);
            modelPaths.unshift(modelPath);
            modelPaths.unshift(superClassPath);
            continue;
        }
        // Create the temporary interface if this model has a super class that will enable us to import the class
        const interfaceFile = getInterfacePath(modelPath);
        if (superClass !== model_1.default.name) {
            const tempObj = fs_1.default
                .readFileSync(getInterfacePath(superClassPath), 'utf8')
                .replace(superClass + 'Obj', 'TempObj')
                .replace('{', '{\n  [key: string]: any;');
            fs_1.default.writeFileSync(interfaceFile, tempObj);
        }
        // Generate the actual interface for the model
        const model = (await (_a = modelPath, Promise.resolve().then(() => __importStar(require(_a))))).default;
        new ts_interface_generator_1.default().generateInterface(model, interfaceFile);
    }
};
const generateInterfaces = async (modelDir) => {
    logger.info(`Generating interfaces for ${modelDir}...`);
    await createInterfaces(modelDir, true);
    await createInterfaces(modelDir, false);
    logger.info(`Finished generating interfaces for ${modelDir}`);
};
exports.default = generateInterfaces;
if (require.main === module) {
    // Read the args
    const args = (0, minimist_1.default)(process.argv.slice(2));
    const { modelDir } = args;
    if (!modelDir) {
        throw Error('--modelDir <file path> is required');
    }
    generateInterfaces(modelDir).catch((err) => logger.error(err));
}
//# sourceMappingURL=generate-interfaces.js.map