import { AggregateOptions, AnyBulkWriteOperation, BulkWriteOptions, BulkWriteResult, CountDocumentsOptions, Db, DeleteOptions, DeleteResult, Document, Filter, FindOptions, ObjectId, UpdateFilter, UpdateOptions, UpdateResult } from 'mongodb';
import CursorIterator from './cursor-iterator';
import { CreateSearchIndex } from './interface';
import MongoIndex from './mongo-index';
import Nongo from './nongo';
export default abstract class Model<T extends {
    _id?: ObjectId;
    revision?: number;
    previousRevision?: number;
    diffs?: any;
    name?: string;
    key?: string;
    environment?: string;
    module?: string;
    deleted?: true;
} = any> {
    nongo: Nongo;
    obj: T;
    isNongoModel: true;
    collection: string;
    db: Db;
    name: string;
    dynamic: boolean;
    private collectionRevision;
    private collectionEnvironmentMapping;
    constructor(nongo: Nongo, obj: T);
    getCollectionRevision(): string;
    protected schemaVersioningEnabled(): boolean;
    getMongoIndexes(): MongoIndex[];
    getAtlasSearchIndexes(): CreateSearchIndex[];
    validate(): Promise<string[]>;
    idString(): string;
    private getLatestRevisionNumber;
    save(): Promise<this>;
    private removeEnvironmentMapping;
    private createEnvironmentMapping;
    private updateEnvironmentMapping;
    revertVersion(id: string, revisionId: string): Promise<this>;
    getRevisionObjForEnvironment(isLive: boolean, id: string): Promise<this>;
    saveEnvironmentRevision(id: string, environmentId: string): Promise<this>;
    removeEnvironmentRevisions(environmentId: string): Promise<DeleteResult>;
    strip(parentPath: any, toStrip: any): void;
    deleteKey(path: any, obj: any): void;
    byId(id: any): Promise<this>;
    getRevisions(query: Filter<Document>, options?: FindOptions<Document>): Promise<this[]>;
    getRevision(query: Filter<Document>): Promise<this>;
    find(query?: Filter<Document>, options?: FindOptions<Document>): Promise<this[]>;
    bulkWrite(operations: AnyBulkWriteOperation<Document>[], options?: BulkWriteOptions): Promise<BulkWriteResult>;
    findIterator(query?: Filter<Document>, options?: FindOptions<Document>, collection?: string): Promise<CursorIterator<this>>;
    findOne(query?: Filter<Document>): Promise<this>;
    remove(query: Filter<Document>, options?: DeleteOptions): Promise<DeleteResult>;
    aggregate(pipeline: Document[], options?: AggregateOptions): Promise<any[]>;
    aggregateIterator(pipeline: Document[], options?: AggregateOptions): Promise<CursorIterator<any>>;
    distinct(field: string, filter?: Filter<Document>): Promise<any[]>;
    update(query: Filter<Document>, updates: UpdateFilter<Document>, options?: UpdateOptions): Promise<UpdateResult<Document>>;
    count(query?: Document, options?: CountDocumentsOptions): Promise<number>;
    abstract defineSchema(): any;
    defineSchemaNoIndex(): any;
    prepareQuery(query: any): any;
    protected defineCollection(): string;
    private throwValidationError;
    /**
     * Recursively remove all the keys form the schema that are used to create indexes.
     *
     * @param parentSchema
     */
    private removeSchemaIndex;
    private toArray;
    private initStructure;
    private initKey;
}
