import { Db, MongoClient } from 'mongodb';
import { Newable } from './interface';
import Model from './model';
import NongoParams from './nongo-params';
import SchemaParser from './schema-parser';
export default class Nongo {
    private params;
    private modelClasses;
    schemaParsers?: SchemaParser[];
    mongoClient?: MongoClient;
    private readonly poolSize?;
    static toObjectId(id: any): any;
    static revisionNumber(revision: string): string;
    schema: any;
    validKeys: any;
    noStripKeys: any;
    db: Db;
    dbName: string;
    static REVISION: string;
    static ENVIRONMENT_MAPPING: string;
    private instances;
    private instanceMap;
    private newGeneratorMap;
    constructor(params: NongoParams, modelClasses: Newable[], schemaParsers?: SchemaParser[], mongoClient?: MongoClient, poolSize?: number);
    connect(): Promise<this>;
    ensureIndexes(): Promise<void>;
    private dropCollection;
    dropDatabase(): Promise<void>;
    /**
     * Remove all documents in the database
     */
    removeAll(): Promise<void>;
    dbExists(): Promise<boolean>;
    /**
     * Used to return the "collection instance" of a {@link Model}. You should use this method to gain access to an object
     * which you can use to run queries again a collection such as {Model.find(..)} etc.
     *
     * @param {{new(...args: any[]): T}} model
     * @returns {T}
     */
    col<T extends Model>(model: new (...args: any[]) => T): T;
    /**
     * Used to create a new instance of a {@link Model}
     *
     * @param {{new(...args: any[]): T}} model
     * @param {T["obj"]} obj
     * @returns {T}
     */
    new<T extends Model>(model: new (...args: any[]) => T, obj: T['obj']): T;
    /**
     * @returns {string} the mongo uri built from {@code this.params}.
     */
    uri(): string;
    private getCollectionNames;
    private ensureCollectionIndexes;
    private waitForIndex;
    private existingIndexes;
    /**
     * Recursively traverse the schema to populate the {@code indexList} with {@link MongoIndex}s constructed using schema
     * kv-pairs.
     *
     * @param parentSchema
     * @param {string} parentPath
     * @param indexSet
     * @returns {Promise<MongoIndex[]>}
     */
    private getSchemaIndexes;
}
