import {MongoClient} from 'mongodb';
import {MongoMemoryServer} from 'mongodb-memory-server';
import {Newable, NongoParams} from '../src';
import Nongo from '../src/nongo';
import Logger from 'wbb-logger';

const logger = new Logger("database-helper.ts");

let mongoMemoryServer: MongoMemoryServer;

const getInMemoryServer = async () => {
  if (!mongoMemoryServer) {
    logger.info("Using in memory server.");
    mongoMemoryServer = await MongoMemoryServer.create();
  }
  return mongoMemoryServer;
};

export default class DatabaseHelper {
  public static DB_NAME = 'testing';
  public static async getNongoInstance(models: Newable[]): Promise<Nongo> {
    return await new Nongo(
      await DatabaseHelper.getNongoParams(),
      models,
    ).connect();
  }

  public static async getNongoParams(): Promise<NongoParams> {
    logger.info("Fetching in memory server.");
      const mongod = await getInMemoryServer();
      return {
        protocol: null,
        uri: mongod.getUri(),
        db: DatabaseHelper.DB_NAME,
      };
  }

  public static async getClient(): Promise<MongoClient> {
    const mongod = await getInMemoryServer();
    return MongoClient.connect(await mongod.getUri());
  }

  public static getDbName(): string {
    return DatabaseHelper.DB_NAME;
  }

  public static async drop(): Promise<void> {
    const client = await DatabaseHelper.getClient();
    const db = client.db(DatabaseHelper.getDbName());
    await db.dropDatabase();
    await client.close();
  }
}
