import Logger from 'wbb-logger';
import { Nongo } from '../src';
import DatabaseHelper from './database-helper';
import DummyModelWithRevisionObj from './models/dummy-model-with-revision-obj';
import DummyModelWithRevision from './models/dummy-model-with-revision.nongo';
import DummyModel from './models/dummy-model.nongo';

const logger = new Logger('model-versioning.test.ts');

const models = [DummyModel, DummyModelWithRevision];

let nongo: Nongo;

// Catch unhanded promise rejections and log them
process.on('unhandledRejection', (reason) => {
  logger.error(reason);
});

describe('Testing nongo revisioning', () => {

  beforeAll(async () => {
    nongo = await DatabaseHelper.getNongoInstance(models);
  })

  afterAll(async () => {
    // remove everything
  })
  
  it('Should create revision collection on DummyModelWithRevision', async () => {
    const collections = await nongo.db.collections();
    const collectionsNames = collections.map(coll => coll.collectionName);
    expect(collectionsNames.length).toEqual(3);
    expect(collectionsNames.includes('DummyModelWithRevisionRevision')).toEqual(true)
    expect(collectionsNames.includes('DummyModelRevision')).toEqual(false)
  })

  it('Should have revision 0.0.1', async () => {
    const obj: DummyModelWithRevisionObj = {
      name: 'John',
      created: new Date(),
    };
    const obj2: DummyModelWithRevisionObj = {
      name: 'Leo',
      age: 35,
      created: new Date(),
    };
    const model = await nongo.new(DummyModelWithRevision, obj).save();
    const model2 = await nongo.new(DummyModelWithRevision, obj2).save();
    expect(model.obj.revision).toEqual('0.0.1');
    expect(model2.obj.revision).toEqual('0.0.1');
  })

  it('Should save revision doc in revision collection', async () => {
    const model = await nongo.col(DummyModelWithRevision).findOne({name: 'John'})
    model.obj.age = 30;
    await model.save();
    expect(model.obj.revision).toEqual('0.0.2');
    const revisions = await nongo.col(DummyModelWithRevision).getRevisions({name: model.obj.name});
    expect(revisions.length).toEqual(1);
  })

  it('Should have 2 revision docs in revision collection', async () => {
    const model = await nongo.col(DummyModelWithRevision).findOne({name: 'John'});
    model.obj.name = 'Nick';
    await model.save();
    const revisions = await nongo.col(DummyModelWithRevision).getRevisions({name: model.obj.name})
    expect(revisions.length).toEqual(2);
    const models = await nongo.col(DummyModelWithRevision).find();
    expect(models.length).toEqual(2);
  })

  it('Should revert to revision 0.0.1', async () => {
    const revision = await nongo.col(DummyModelWithRevision).getRevision({name: "John", revision: "0.0.1"});
    expect(revision.obj.revision).toEqual("0.0.1");
    const model = await nongo.col(DummyModelWithRevision).findOne({name: 'Nick'});
    const latestDoc = await nongo.col(DummyModelWithRevision).revertVersion(model.obj._id);
    expect(latestDoc.obj.name).toEqual(revision.obj.name)
  })
})